class LearningObjectBuilder
  def self.build_from_dspace(item)
    lo = LearningObject.new(
      name: item.name,
      id_dspace: item.id,
      thumbnail: nil,
      metadata: dspace_metadata_to_hash(item.metadata)
    )

    institution = lo.get_metadata_value_of('dc.creator')
    institution = 'Desconhecido' if institution.blank?
    lo.publisher = Institution.where(name: institution).first_or_create

    object_type = lo.get_metadata_value_of('dc.type')
    object_type = 'Desconhecido' if object_type.blank?
    lo.object_type = ObjectType.where(name: object_type).first_or_create

    date = lo.get_metadata_value_of('dc.date.issued')
    lo.published_at = Time.iso8601(date) unless date.nil?

    code = lo.get_metadata_value_of('dc.language')
    code = 'pt' if code.blank?
    language = Language.where(code: code).first
    language = Language.create(name: code, code: code) if language.nil?
    lo.language = language

    lo.author = lo.get_metadata_values_of('dc.contributor.author').join(', ')
    lo.description = lo.get_metadata_value_of('dc.description')

    lo.state = LearningObject.states[:published]

    lo
  end

  def self.dspace_metadata_to_hash(metadata)
    hash = Hash.new { |h, k| h[k] = Array.new(&h.default_proc) }
    metadata.each { |m| hash[m.key] << m.value }
    hash
  end
end
