module Stageable
  extend ActiveSupport::Concern

  included do
    has_many :educational_stages, through: :stage_relations, dependent: :destroy
    has_many :stage_relations, as: :stageable, dependent: :destroy
  end
  
  
  # ~~~~ stageable actions ~~~~ #
  # A stageable can add educational stages
  # Examples:
  # stageable.add_educational_stages(ids: [1, 2, 3])
  # stageable.add_educational_stages(ids: [1])
  def add_educational_stages(ids: [])
    ids.each do |educational_stage_id|
      educational_stage = EducationalStage.find(educational_stage_id)
      StageRelation.where(educational_stage: educational_stage, stageable: self).first_or_create
    end
  end

  # A stageable can remove educational stages
  # Examples:
  # stageable.remove_educational_stages(ids: [1, 2, 3])
  def remove_educational_stages(ids: [])
    ids.each do |educational_stage_id|
      educational_stage = EducationalStage.find(educational_stage_id)
      StageRelation.where(educational_stage: educational_stage, stageable: self).destroy_all
    end
  end

  # ~~~~ end stageable actions ~~~~ #



end
