# == Schema Information
#
# Table name: user_categories
#
#  id         :integer          not null, primary key
#  name       :string
#  reference  :float
#  created_at :datetime         not null
#  updated_at :datetime         not null
#

class UserCategory < ApplicationRecord
  has_many :score_user_categories
  has_many :scores, through: :score_user_categories

  validates_presence_of :name
  validates_uniqueness_of :name

  def calculate_reference
    hash = reputation_values

    return 0 unless hash.values.all? { |x| !x.nil? }

    update(reference: ScoreCalculatorService.new(User.new).calculate(hash))

    # defining maximum reference change every reference in UserCategory
    UserCategory.where.not(id: id).each(&:calculate_reference) if reference == UserCategory.maximum(:reference)
  end

  private

  # return a hash with 'score code' => 'score_user_category maximum value'
  def reputation_values
    Score.object_type('User').includes(:score_user_categories).map { |s| [s.code, s.score_user_categories.find { |sc| sc.score_id == s.id && sc.user_category_id == id }.value] }.to_h
  end
end
