module SearchService
  class Collection < Model
    def search
      ::Collection.search(@search.query, where: { privacy: 'public' }, order: order_hash, page: @search.page, per_page: @search.results_per_page)
    end

    def autocomplete
      params = {  where: { privacy: 'public' },
                  fields: ['name^10', 'description', 'owner'] }
      result = ::Collection.search(@search.query, autocomplete_params.merge(params))

      thumbnail = proc { |_obj| '/assets/icons/collection' }
      type = proc { |_obj| 'Collection' }
      autocomplete_render(result, type, thumbnail)
    end

    private

    def order_hash
      case @search.order
      when 'author' then { owner: { order: :asc, unmapped_type: :string } }
      when 'publicationasc' then { created_at: { order: :asc, unmapped_type: :timestamp } }
      when 'publicationdesc' then { created_at: { order: :desc, unmapped_type: :timestamp } }
      when 'title' then { name: { order: :asc, unmapped_type: :string } }
      else { score: { order: :desc, unmapped_type: :integer } }
      end
    end
  end
end
