namespace :score do
  desc 'calculate score async'
  task calculate_all: :environment do
    %w(LearningObject Collection User).map { |o| calculate_async(o) }
  end

  desc 'calculate score sync'
  task :calculate_sync, [:klass] => :environment do |_t, args|
    return false unless %w(User LearningObject Collection).include? args.klass
    klass = args.klass.constantize
    progressbar = ProgressBar.create(title: 'calculate LearningObjects score', total: klass.count)
    Parallel.each(klass.all, in_threads: 8) do |lo|
      ActiveRecord::Base.connection_pool.with_connection do
        calculate_sync(lo)
      end
      progressbar.increment
    end
  end

  private

  def calculate_async(object)
    return false unless %w(Collection LearningObject User).include?(object)

    object.constantize.all.find_each do |item|
      ScoreCalculatorWorker.perform_async(item.id, item.class.name)
    end
  end

  def calculate_sync(object)
    return false unless %w(Collection LearningObject User).include?(object.class.name)

    calculator = ScoreCalculatorService.new(object)

    score = calculator.calculate

    object.update(score: score) if score.is_a? Float

    object.update(user_category_id: calculator.category_by_reputation) if object.class.name == 'User'
  end
end
