require 'test_helper'

class SearchServiceTest < ActiveSupport::TestCase
  test 'fetch learning object with all search params' do
    reindex LearningObject

    search = SearchService.search(Search.new(los_complete_search), users(:john))
    search.each {|o| assert_instance_of LearningObject, o}
  end

  test 'fetch all public collections' do
    reindex Collection

    search = SearchService.search(Search.new(collections_search), users(:john))
    search.each {|o| assert_instance_of Collection, o}
  end

  test 'fetch users named john' do
    reindex User

    search = SearchService.search(Search.new(users_john_search), users(:john))
    assert_equal [users(:john), users(:one)], search
  end

  test 'autocomplete users named john' do
    reindex User

    search = SearchService.autocomplete(Search.new(users_john), users(:john))
    assert_equal users_autocomplete_hashes([users(:john), users(:one)]), search
  end

  private

  def los_complete_search
    default_params.merge(
      query: 'teste',
      search_class: 'LearningObject',
      tags: 'Matemática',
      types: 'Imagem'
    )
  end

  def collections_search
    default_params.merge(search_class: 'Collection')
  end

  def users_john_search
    default_params.merge(users_john)
  end

  def users_john
    {
      query: 'John',
      search_class: 'User'
    }
  end

  def default_params
    {
      page: 1,
      results_per_page: 10,
      order: 'score'
    }
  end

  def reindex(klass)
    klass.reindex
    klass.searchkick_index.refresh
  end

  def users_autocomplete_hashes(users = [])
    users.map do |user|
      {
        id: user.id,
        name: user.name,
        type: 'User',
        thumbnail: user.avatar.url(:thumb)
      }
    end
  end
end
